/* Gnofract -- a little fractal generator-browser program
 * Copyright (C) 1999 2000  Aurelien Alleaume
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gnome.h>
#include "fract.h"

extern int calc_num;

inline int scan_double(struct param_image param, struct rgb tmp[], int my_calc_num);
inline int scan_double_aa(struct param_image param, struct rgb tmp[], int my_calc_num);


int calc(struct param_image *param, int my_calc_num)
{
	struct rgb *tmp;

	tmp = param->tmp_img;

	/*
	   * Si l'image n'a pas le bon format(en fonction de Xres et Yres), 
	   * on modifie ses caracteristiques : on augmente largeur ou hauteur
	 */

	if ((double) param->Xres / (double) param->Yres <
	    (param->xmax - param->xmin) / (param->ymax - param->ymin)) {
		double centery, hauteur_new;
		centery = (param->ymax + param->ymin) / 2;
		hauteur_new =
		    (param->xmax - param->xmin) * ((double) param->Yres / (double) param->Xres);
		param->ymax = centery + hauteur_new / 2;
		param->ymin = centery - hauteur_new / 2;

	} else if ((double) param->Xres / (double) param->Yres >
		   (param->xmax - param->xmin) / (param->ymax - param->ymin)) {
		double centerx, largeur_new;
		centerx = (param->xmax + param->xmin) / 2;
		largeur_new =
		    (param->ymax - param->ymin) * ((double) param->Xres / (double) param->Yres);
		param->xmax = centerx + largeur_new / 2;
		param->xmin = centerx - largeur_new / 2;
	}


	if (param->aa_profondeur > 1)
		scan_double_aa(*param, tmp, my_calc_num);
	else
		scan_double(*param, tmp, my_calc_num);

	return 0;
}


inline int scan_double(struct param_image param, struct rgb tmp[], int my_calc_num)
{

	int w = 0, i = 0, j = 0;
	double deltax = 0, deltay = 0, x = 0, y = 0;


	gtk_stuff_init();

	deltax = (param.xmax - param.xmin) / param.Xres;
	deltay = (param.ymax - param.ymin) / param.Yres;
	y = param.ymin;
	for (i = 0; i < param.Yres; i++) {
		x = param.xmin;
		gtk_stuff((gfloat) i / (gfloat) param.Yres, i);
		if (param.fractal_type == 0)
			for (j = 0; j < param.Xres; j++) {
				if (my_calc_num < calc_num)
					return -1;	/* test if another calc has been started */
				tmp[w] =
				    test_mandelbrot(x, y, param.eject_val, param.nbit_max,
						    param.cmap);
				w++;
				x = deltax + x;
		} else
			for (j = 0; j < param.Xres; j++) {
				if (my_calc_num < calc_num)
					return -1;	/* test if another calc has been started */
				tmp[w] =
				    test_fonction(x, y, param.eject_val, param.nbit_max,
						  param.cmap);
				w++;
				x = deltax + x;
			}
		y = y + deltay;
	}
	gtk_stuff((gfloat) i / (gfloat) param.Yres, i);
	gtk_stuff_fin();

	return 0;
}



inline int scan_double_aa(struct param_image param, struct rgb tmp[], int my_calc_num)
{

	int w = 0, i = 0, j = 0, k = 0, l = 0;
	double deltax = 0, deltay = 0, x = 0, y = 0, delta_aa_x = 0, delta_aa_y = 0;
	struct rgb *pixeltab;
	int depth = param.aa_profondeur;
	unsigned int pixel_r_val = 0, pixel_g_val = 0, pixel_b_val = 0;


	pixeltab = malloc(depth * depth * sizeof(struct rgb));

	gtk_stuff_init();
	deltax = (param.xmax - param.xmin) / param.Xres;
	deltay = (param.ymax - param.ymin) / param.Yres;
	delta_aa_x = deltax / depth;
	delta_aa_y = deltay / depth;

	y = param.ymin;
	for (i = 0; i < param.Yres; i++) {
		x = param.xmin;
		gtk_stuff((gfloat) i / (gfloat) param.Yres, i);
		for (j = 0; j < param.Xres; j++) {
			double ytmp, xtmp;
			int wtmp = 0;
			if (my_calc_num < calc_num)
				return -1;	/* test if another calc has been started */
			pixel_r_val = 0;
			pixel_g_val = 0;
			pixel_b_val = 0;
			ytmp = y - (depth / 2) * delta_aa_y;
			for (k = 0; k < depth; k++) {
				xtmp = x - (depth / 2) * delta_aa_x;
				if (param.fractal_type == 0)
					for (l = 0; l < depth; l++) {
						pixeltab[wtmp]
						    = test_mandelbrot(xtmp,
								      ytmp,
								      param.eject_val,
								      param.nbit_max, param.cmap);
						wtmp++;
						xtmp += delta_aa_x;
				} else
					for (l = 0; l < depth; l++) {
						pixeltab[wtmp]
						    = test_fonction(xtmp,
								    ytmp, param.eject_val,
								    param.nbit_max, param.cmap);
						wtmp++;
						xtmp += delta_aa_x;
					}
				ytmp += delta_aa_y;
			}


			for (k = 0; k < depth * depth; k++) {
				pixel_r_val += (pixeltab[k]).r;
				pixel_g_val += (pixeltab[k]).g;
				pixel_b_val += (pixeltab[k]).b;
			}

			tmp[w].r = pixel_r_val / (depth * depth);
			tmp[w].g = pixel_g_val / (depth * depth);
			tmp[w].b = pixel_b_val / (depth * depth);
			w++;
			x = deltax + x;
		}
		y += deltay;
	}

	gtk_stuff((gfloat) i / (gfloat) param.Yres, i);
	gtk_stuff_fin();
	free(pixeltab);
	return 0;
}
