/* Gnofract -- a little fractal generator-browser program
 * Copyright (C) 1999 2000  Aurelien Alleaume
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#include <gnome.h>
#include "fract.h"
#include "support.h"
#include <dlfcn.h>

extern int calc_num;		/* we will know if another calc has been started */
extern int calculating;
extern GtkWidget *drawing_area;
extern GtkWidget *appbar1;
extern GtkWidget *app;
extern GtkWidget *propertybox;
extern struct param_image param_current, param_defaut;
extern double zoom_plus, zoom_moins;

void realloc_memory(struct param_image *param)
{
	param->tmp_img =
	    (struct rgb *) realloc(param->tmp_img,
				   (param->Xres) * (param->Yres) * sizeof(struct rgb));
}

void reset_coord(void)
{
	param_current.xmin = param_defaut.xmin;
	param_current.ymin = param_defaut.ymin;
	param_current.xmax = param_defaut.xmax;
	param_current.ymax = param_defaut.ymax;
}


void init_cmap(struct param_image *param)
{
	int i;
	struct rgb *cmap;
	strcpy(param_current.cmap_file_name, ".");
	param->cmap_flag = 0;
	if (param->cmap == NULL)
		param->cmap = (struct rgb *) malloc(sizeof(struct rgb) * 256);
	cmap = param->cmap;
	for (i = 0; i < 256; i++) {
		cmap[i].r = 10 * i;
		cmap[i].g = i;
		cmap[i].b = i;
	}
}

int load_cmap(struct param_image *param)
{
	FILE *file;
	struct rgb *cmap;
	int i;
	if ((file = fopen(param->cmap_file_name, "rb")) == NULL) {
		gtk_widget_show(gnome_warning_dialog
				(_("Colormap not loaded : I can't open the file  !")));
		return FALSE;
	}
	param->cmap_flag = 1;
	cmap = param->cmap;
	for (i = 0; i < 256; i++) {
		char line[160];
		
int r, g, b;
		if (fgets(line, 100, file) == NULL)
			
break;
		
sscanf(line, "%u %u %u", &r, &g, &b);
		
cmap[i].r = (unsigned char) r;
		cmap[i].g = (unsigned char) g;
		cmap[i].b = (unsigned char) b;
	}
	fclose(file);
	return TRUE;
}

int load_formula(struct param_image *param)
{
	static void *handle = NULL;
	if (handle != NULL) {
		unload_plugin();
		dlclose(handle);
	}
	handle = dlopen(param->formula_file_name, RTLD_NOW);
	if (dlerror() != NULL) {
		gtk_widget_show(gnome_warning_dialog(_("Not loaded : I can't load the module  !")));
		return FALSE;
	}
	test_fonction = dlsym(handle, "test_fonction");
	if (dlerror() != NULL) {
		gtk_widget_show(gnome_warning_dialog
				(_("Not loaded : I can't find the test function !")));
		return FALSE;
	}
	init_plugin = dlsym(handle, "init_plugin");
	if (dlerror() != NULL) {
		gtk_widget_show(gnome_warning_dialog
				(_("Not loaded : I can't find the init function !")));
		return FALSE;
	}

	unload_plugin = dlsym(handle, "unload_plugin");
	if (dlerror() != NULL) {
		gtk_widget_show(gnome_warning_dialog
				(_("Not loaded : I can't find the unload function !")));
		return FALSE;
	}

	init_plugin();
	param->fractal_type = 1;
	return TRUE;
}

inline void redraw_image_zone(GtkWidget * widget, int x, int y, int width, int height,
			      int image_width, struct rgb *img)
{
	gdk_draw_rgb_image(widget->window,
			   widget->style->white_gc,
			   x, y, width, height,
			   GDK_RGB_DITHER_NONE, (unsigned char *) img + (x + y * image_width) * 3,
			   image_width * 3);
}

void refresh_image(void)
{
	calc_num++;		/* if another calc is running it will stop soon */
	calculating = TRUE;
	property_box_refresh();
	calc(&param_current, calc_num);
	calculating = FALSE;
	show_image(param_current);

}


inline void gtk_stuff_init(void)
{
	gnome_appbar_set_progress((GnomeAppBar *) appbar1, (gfloat) 0);
	gnome_appbar_push((GnomeAppBar *) appbar1, _(" calculating ....  "));
}

inline void gtk_stuff(gfloat pourcentage, int y)
{
	gnome_appbar_set_progress((GnomeAppBar *) appbar1, pourcentage);
	redraw_image_zone(drawing_area, 0, y - 1, param_current.Xres, 1, param_current.Xres,
			  param_current.tmp_img);
	while (gtk_events_pending())
		gtk_main_iteration();
}

inline void show_image(struct param_image param)
{
	if (param_current.tmp_img != NULL 
)
		redraw_image_zone(drawing_area, 0, 0,
				  param_current.Xres, param_current.Yres, param_current.Xres,
				  param_current.tmp_img);
}


inline void gtk_stuff_fin(void)
{
	gnome_appbar_clear_stack((GnomeAppBar *) appbar1);
	gnome_appbar_set_progress((GnomeAppBar *) appbar1, 0.0);
}

void property_box_refresh(void)
{

	if (propertybox != NULL) {
		gtk_entry_set_text(GTK_ENTRY
				   (lookup_widget
				    (propertybox,
				     "combo_entry_image_width")), g_strdup_printf("%d",
										  param_current.
										  Xres));
		gtk_entry_set_text(GTK_ENTRY
				   (lookup_widget(propertybox, "combo_entry_image_height")),
				   g_strdup_printf("%d", param_current.Yres));
		gtk_entry_set_text(GTK_ENTRY
				   (lookup_widget(propertybox, "combo_entry_nb_iteration_max")),
				   g_strdup_printf("%d", param_current.nbit_max));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_eject_val")),
				   g_strdup_printf("%d", param_current.eject_val));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_xmin")),
				   g_strdup_printf("%.25f", param_current.xmin));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_ymin")),
				   g_strdup_printf("%.25f", param_current.ymin));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_xmax")),
				   g_strdup_printf("%.25f", param_current.xmax));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_ymax")),
				   g_strdup_printf("%.25f", param_current.ymax));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_zoom_plus")),
				   g_strdup_printf("%.20f", zoom_plus));
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(propertybox, "combo_entry_zoom_moins")),
				   g_strdup_printf("%.20f", zoom_moins));
	}

}
